package com.om.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.om.entity.dto.*;
import com.om.entity.po.*;
import com.om.entity.vo.AppRepairQueryPageVO;
import com.om.entity.vo.DiagnosticReportVO;
import com.om.entity.vo.RepairCreateVO;
import com.om.exception.BadReqException;
import com.om.exception.BizException;
import com.om.mapper.RepairMapper;
import com.om.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.om.utils.Result;
import com.om.utils.UserContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author bmmx
 * @since 2024-02-18
 */
@Service
public class RepairServiceImpl extends ServiceImpl<RepairMapper, Repair> implements IRepairService {

    @Resource
    private IFaultCodeService faultCodeService;

    @Resource
    private IEcuInfoService ecuInfoService;

    @Resource
    private IElectronicControlService electronicControlService;

    @Resource
    RepairMapper repairMapper;

    @Resource
    IUserService userService;

    @Override
    @Transactional
    public Result AppCreateReport(APPReportCreateDTO dto) {
        Integer userId = UserContext.getUserId();
        // 判断数据、
        if (BeanUtil.isEmpty(dto)) {
            throw new BadReqException("数据为空");
        }

        Repair repair = BeanUtil.copyProperties(dto, Repair.class);
        List<AppElectronicDTO> electronicControls = dto.getElectronicControls();

        List<ElectronicControl> electronicControlList = new ArrayList<>();
        for (AppElectronicDTO electronicControlDTO : electronicControls) {
            ElectronicControl electronicControl = BeanUtil.copyProperties(electronicControlDTO, ElectronicControl.class);
            electronicControlList.add(electronicControl);
            List<AppEcuInfoDTO> ecuInfors = electronicControlDTO.getEcuInfors();
            List<AppFaultCodeDTO> faultCodes = electronicControlDTO.getFaultCodes();

            List<EcuInfo> ecuInfos = BeanUtil.copyToList(ecuInfors, EcuInfo.class);
            List<FaultCode> faultCodes1 = BeanUtil.copyToList(faultCodes, FaultCode.class);

            for (EcuInfo ecuInfo : ecuInfos) {
                ecuInfo.setElectronicControlId(electronicControl.getId());
            }

            for (FaultCode faultCode : faultCodes1) {
                faultCode.setElectronicControlId(electronicControl.getId());
            }
            
            repair.setElectronicControlId(electronicControl.getId());
            ecuInfoService.saveBatch(ecuInfos);
            faultCodeService.saveBatch(faultCodes1);

        }
        electronicControlService.saveBatch(electronicControlList);
        repair.setUserId(userId);
        this.save(repair);
        return Result.ok();
    }

    @Override
    public Result<AppRepairQueryPageVO> AppGetPageList(APPRepairQueryPageDTO dto) {
        Integer userId = UserContext.getUserId();
        if (userId==null){
            throw new BizException("用户不存在");
        }
        //获取数据
        Integer pageIndex = dto.getPageIndex();
        Integer pageSize = dto.getPageSize();

        //分页查询
        Page<Repair> page = this.lambdaQuery()
                .orderByDesc(Repair::getCreateTime)
                .eq(Repair::getUserId,userId)
                .page(new Page<>(pageIndex, pageSize));

        if (BeanUtil.isEmpty(page)) {
            throw new BadReqException("请求失败");
        }

        AppRepairQueryPageVO vo = new AppRepairQueryPageVO();

        vo.setSize(pageSize);
        vo.setCurrent(pageIndex);
        vo.setPages((int) page.getPages());
        vo.setTotal((int) page.getTotal());

        List<Repair> records = page.getRecords();
        if (CollectionUtil.isEmpty(records)) {
            records = Collections.emptyList();
        }

        List<DiagnosticReportVO> list = new ArrayList<>();
        for (Repair record : records) {
            DiagnosticReportVO diagnosticReportVO = new DiagnosticReportVO();
            diagnosticReportVO.setCreationTime(LocalDateTime.now());
            diagnosticReportVO.setDiagnosticReportId(record.getId());
            diagnosticReportVO.setDiagnosticReportName(record.getMainTitle());
            diagnosticReportVO.setFileName(record.getFileName());
            Integer electronicControlId = record.getElectronicControlId();

            List<FaultCode> faultCodeList = faultCodeService.lambdaQuery()
                    .eq(FaultCode::getElectronicControlId, electronicControlId)
                    .list();

            diagnosticReportVO.setFaultCodesNumber(String.valueOf(faultCodeList.size()));

            list.add(diagnosticReportVO);
        }

        vo.setRecords(list);
        return Result.ok(vo);
    }

    @Override
    @Transactional
    public Result AppDelete(Integer id) {
        Integer userId = UserContext.getUserId();
        if (userId==null){
            throw new BizException("用户不存在");
        }
        //根据id查询
        Repair repair = this.getById(id);
        if (BeanUtil.isEmpty(repair)) {
            throw new BadReqException("该诊断报告不存在");
        }
        if (!Objects.equals(repair.getUserId(), userId)){
            throw new BadReqException("不能删除他人的诊断报告");
        }
        Integer electronicControlId = repair.getElectronicControlId();

        List<FaultCode> faultCodeList = faultCodeService.lambdaQuery()
                .eq(FaultCode::getElectronicControlId, electronicControlId)
                .list();
        List<Integer> faultCodeIds = faultCodeList.stream().map(c -> c.getId()).collect(Collectors.toList());


        List<EcuInfo> ecuInfos = ecuInfoService.lambdaQuery()
                .eq(EcuInfo::getElectronicControlId, electronicControlId)
                .list();

        List<Integer> ecuIds = ecuInfos.stream().map(c -> c.getId()).collect(Collectors.toList());
        faultCodeService.removeByIds(faultCodeIds);
        ecuInfoService.removeByIds(ecuIds);

        electronicControlService.removeById(electronicControlId);
        this.removeById(id);
        return Result.ok();
    }






    @Override
    public Result create(RepairDTO repairDTO) {

        Repair repair = new Repair();
        int eleId = new Random(100000000).nextInt();
        repair.setElectronicControlId(eleId);
        BeanUtil.copyProperties(repairDTO,repair);
        boolean repairBoolean = this.save(repair);

        ElectronicControl ele = new ElectronicControl();
        ele.setId(eleId);
        ele.setElectronicControlName(repairDTO.getElectronicControls().getElectronicControlName());
        BeanUtil.copyProperties(repairDTO.getReportTitleDto(),ele);
        boolean eleBoolean = electronicControlService.save(ele);

        EcuInfo ecuInfo = new EcuInfo();
        BeanUtil.copyProperties(repairDTO.getElectronicControls().getEcuInfo(),ecuInfo);
        ecuInfo.setElectronicControlId(eleId);
        boolean ecuBoolean = ecuInfoService.save(ecuInfo);

        FaultCode faultCode = new FaultCode();
        BeanUtil.copyProperties(repairDTO.getElectronicControls().getFaultCode(),faultCode);
        faultCode.setElectronicControlId(eleId);
        boolean fauBoolean = faultCodeService.save(faultCode);

        //此处需要做一个事务回滚
        //如果四个有一个添加失败，则进行事务回滚

        //如果没有，则证明成功
        Integer userId = UserContext.getUserId();
        User user = userService.getById(userId);
        RepairCreateVO createVO = new RepairCreateVO();
        BeanUtil.copyProperties(user,createVO);


        //这里需要做个返回
        //出现问题，所指的地址分别是啥
        return Result.ok();


    }

    @Override
    public Result delete(Integer id) {
        Repair repair = this.getById(id);
        if (repair == null){
            throw new BadReqException("删除错误");
        }
        this.removeById(id);
        Integer eleId = repair.getElectronicControlId();
        electronicControlService.removeById(id);

        QueryWrapper<EcuInfo> ecuQueryWrapper = new QueryWrapper<EcuInfo>();
        QueryWrapper<FaultCode> fauQueryWrapper = new QueryWrapper<FaultCode>();
        ecuQueryWrapper.lambda().eq(EcuInfo::getElectronicControlId,eleId);
        fauQueryWrapper.lambda().eq(FaultCode::getElectronicControlId,eleId);
        removeById(id);
        ecuInfoService.remove(ecuQueryWrapper);
        faultCodeService.remove(fauQueryWrapper);


        return Result.ok();
    }

    @Override
    public Result getByIds(Integer reportId) {
        Repair repair = this.getById(reportId);
        if (repair == null){
            throw new BadReqException("查询错误");
        }
        Integer eleId = repair.getElectronicControlId();
        ElectronicControl ele = electronicControlService.getById(eleId);
        QueryWrapper<EcuInfo> ecuQueryWrapper = new QueryWrapper<>();
        QueryWrapper<FaultCode> fauQueryWrapper = new QueryWrapper<>();
        ecuQueryWrapper.lambda().eq(EcuInfo::getElectronicControlId,eleId);
        fauQueryWrapper.lambda().eq(FaultCode::getElectronicControlId,eleId);
        EcuInfo ecu = ecuInfoService.getOne(ecuQueryWrapper);
        FaultCode faultCode = faultCodeService.getOne(fauQueryWrapper);

        RepairCreateVO repairCreateVO = new RepairCreateVO();
        BeanUtil.copyProperties(repair,repairCreateVO);
        BeanUtil.copyProperties(ecu,repairCreateVO.getElectronicControls().getEcuInfo());
        BeanUtil.copyProperties(faultCode,repairCreateVO.getElectronicControls().getFaultCode());
        repairCreateVO.getElectronicControls().setElectronicControlName(ele.getElectronicControlName());
        return Result.ok(repairCreateVO);
    }

    @Override
    public Result getPageList(RepairDTO repairDTO) {
        Integer userId = UserContext.getUserId();
        User user = userService.getById(userId);
        QueryWrapper<Repair> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Repair::getUserId,userId);
        List<Repair> repairs = repairMapper.selectList(queryWrapper);

        return Result.ok(repairs);
    }
}

